! FileSystem Functions

! Escape shell meta character helper
With[{meta = Characters[StringJoin["\\", " ", "\"'", "*?[", "$", "(){}<>!|&", ";"]]},
  With[{replace = Map[(#->"\\"//#)&, meta]},
    EscapeShellMetaCharacter[s_String] := StringReplace[s, replace];
    ]];

! Helper Function
$FileOperation[op_String, files__String] := With[{
  cmd = StringJoin@@Rest[Flatten[Thread[{" ", Prepend[
    If[#[1] === "-", "./"//EscapeShellMetaCharacter[#], EscapeShellMetaCharacter[#]]&/@{files}, op]}]]]},
!!  Print["System: [", cmd, "]"]; Return[$Failed]; (* for Debug *)
  If[0 <=> System[cmd], {False, $Failed}, {True, RealPath[Last[{files}]]}]];

Protect[EscapeShellMetaCharacter, $FileOperation];

! Mathematica Compatible Functions

! Name Space Operator
FileNames[] := Module[{fp, ret, tmp1, tmp2},
  fp = OpenRead["!ls -d 2>/dev/null"];
  If[fp === $Failed, {},
    tmp1 = Table[Check[Read[fp, String], EndOfFile], {1000}];
    tmp2 = Select[tmp1, # <=> EndOfFile&];
    ret = tmp2;
    While[Length[tmp1] === Length[tmp2],
      tmp1 = Table[Check[Read[fp, String], EndOfFile], {1000}];
      tmp2 = Select[tmp1, # <=> EndOfFile&];
      ret = Join[ret, tmp2]];
    ret]];
FileNames[form_String] := Select[FileNames[], StringMatchQ[#, form]&];
FileNames[{forms___String}] := With[{list = FileNames[]},
  Map[With[{form = #}, Select[list, StringMatchQ[#, form]&]]&, {forms}]];

DirectoryName[dir_String, n_Real] := Module[{dir$ = dir},
  Do[dir$ = DirectoryName[dir$], {i, n}]; dir$];
DirectoryName[dir_String] := With[{pos = StringPosition[dir[1,-2], "/"]},
  If[pos === {}, "", dir[1, pos[[-1, 2]]]]];

ToFileName[dir_String, file_String] := ToFileName[{dir}, file];
ToFileName[{dirs__String}, file_String] := StringJoin[ToFileName[{dirs}],
  If[file[1] === "/", file[2,-1], file]];
ToFileName[{dirs__String}] := With[{tmp = Select[{dirs}, # <=> ""&, 1],
  path = Select[Map[
    #[If[#[1] === "/", 2, 1], If[#[-1] === "/", -2, -1]]&, {dirs}], # <=> ""&]},
  If[tmp === {}, "", If[First[tmp][1] === "/", "/", ""]]//StringJoin@@Flatten[Thread[{path, "/"}]]];

Protect[FileNames, DirectoryName, ToFileName];

! Directory Operator
$HomeDirectory := Environment$["HOME"];
CreateDirectory[path_String] := Last[$FileOperation["mkdir -p", path]];
DeleteDirectory[path_String, opt___Rule] := If[First[$FileOperation[
  If[True === DeleteContents/.Override[opt, DeleteContents->False],
    "rm -r", "rmdir"], path]], Null, $Failed];
RenameDirectory[src_String, dst_String] :=
  If[Not[DirectoryQ[src]] || DirectoryQ[dst], $Failed,
    Last[$FileOperation["mv", src, dst]]];
CopyDirectory[src_String, dst_String] :=
  If[Not[DirectoryQ[src]] || DirectoryQ[dst], $Failed,
    Last[$FileOperation["cp -R", src, dst]]];

SetAttrubutes[{$HomeDirectory}, Constant];
Protect[CreateDirectory, DeleteDirectory, RenameDirectory, CopyDirectory];

! File Operator
CopyFile[src_String, dst_String] := If[Not[FileQ[src]]
  || RealQ[FromFileDate[dst]], $Failed, Last[$FileOperation["cp", src, dst]]];
RenameFile[src_String, dst_String] := If[Not[FileQ[src]]
  || RealQ[FromFileDate[dst]], $Failed, Last[$FileOperation["mv", src, dst]]];
DeleteFile[ file_String]    := If[First[$FileOperation["rm", file ]], Null, $Failed];
DeleteFile[{files__String}] := If[First[$FileOperation["rm", files]], Null, $Failed];

SetFileDate[path_String,
  {CCYY_Real, MM_Real, DD_Real, hh_Real, mm_Real, SS_Real}] := With[{
    date = DateString[{CCYY, MM, DD, hh, mm, SS}]},
    If[Not[RealQ[FromFileDate[path]]], $Failed, If[0 <=>
      System["touch -m -t "//date[7,10]//date[1,2]//date[4,5]
	//date[12,13]//date[15,16]//"."//date[18,19]//" "
	  //EscapeShellMetaCharacter[path]//" 2>/dev/null"], $Failed, Null]]];

Protect[CopyFile, RenameFile, DeleteFile, SetFileDate];

! File Stat
FileDate[path_String] := With[{t = FromFileDate[path]},
  If[RealQ[t], ToDate[t], $Failed]];

FileType[path_String] := If[RealQ[FromFileDate[path//"/"]], Directory,
  If[RealQ[FromFileDate[path]], File, None]];

FileByteCount[path_String] := Module[{fp, tmp},
  If[Not[FileQ[path]], $Failed,
    fp = OpenRead["!wc -l "//EscapeShellMetaCharacter[path]//" 2>/dev/null"];
    If[fp === $Failed, $Failed,
      tmp = Check[Read[fp, Real], $Failed]; Close[fp]; tmp]]];

Protect[FileDate, FileType, FileByteCount];

! SAD Extension Function
DirectoryQ[_] := False;
FileQ[_] := False;
DirectoryQ[path__] := DirectoryQ/@{path};
FileQ[path__] := FileQ/@{path};
DirectoryQ[path_String] := With[{$path = RealPath[path]}, And[StringQ[$path], RealQ[FromFileDate[$path//"/"]]]];
FileQ[path_String] := With[{$path = RealPath[path]}, And[StringQ[$path], RealQ[FromFileDate[$path]], Not[RealQ[FromFileDate[$path//"/"]]]]];

Protect[FileQ, DirectoryQ];

! TemporaryName for backward compatibility[insecure/not recommended]
If[Not[FBoundQ$[TemporaryName]], With[{padchar = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz",
  host = With[{p = StringPosition[$MachineName, "."]}, If[p <=> {}, $MachineName[1, p[[1, 1]] - 1], $MachineName]]},

  $TemporaryNameCount = 0;
  TemporaryName[___] := StandardForm[With[{prefix = "/tmp/tmp."//$TemporaryNameCount//".",
    suffix = StringFill[ToString[GetPID[]], "0", -8]//host},
    If[$TemporaryNameCount == 0,
      Print["Warning: TemporaryName[] possibly used unsafely; consider using MkSecureTemp[]"]];
    Unprotect[$TemporaryNameCount]; ++$TemporaryNameCount; Protect[$TemporaryNameCount];
    Do[With[{fn = StringJoin[prefix, Null@@padchar/@Floor[1 + StringLength[padchar] * Random[6]], suffix]},
      If[Not[RealQ[FromFileDate[fn]]], Return[fn]]], {Infinity}]]];
  Protect[$TemporaryNameCount, TemporaryName]]];

! End of File
