(* Encode List of Unicode code-points into UTF-8 character String *)
Unicode2UTF8[u___Real] := FromCharacterCode[Map[Which[
  # =<       0x7F,		Null[#],

  # =<      0x7FF,		Null[
    BitwiseOr[0xc0, BitwiseAnd[0x1f, Floor[# / 2^6]]],
    BitwiseOr[0x80, BitwiseAnd[0x3f, Floor[#]]]],

  # =<     0xFFFF,		Null[
    BitwiseOr[0xe0, BitwiseAnd[0x0f, Floor[# / 2^12]]],
    BitwiseOr[0x80, BitwiseAnd[0x3f, Floor[# / 2^6]]],
    BitwiseOr[0x80, BitwiseAnd[0x3f, Floor[#]]]],

  # =<  0x1FFFFF,		Null[
    BitwiseOr[0xf0, BitwiseAnd[0x07, Floor[# / 2^18]]],
    BitwiseOr[0x80, BitwiseAnd[0x3f, Floor[# / 2^12]]],
    BitwiseOr[0x80, BitwiseAnd[0x3f, Floor[# / 2^6]]],
    BitwiseOr[0x80, BitwiseAnd[0x3f, Floor[#]]]],

  # =<  0x3FFFFFF,		Null[
    BitwiseOr[0xf8, BitwiseAnd[0x03, Floor[# / 2^24]]],
    BitwiseOr[0x80, BitwiseAnd[0x3f, Floor[# / 2^18]]],
    BitwiseOr[0x80, BitwiseAnd[0x3f, Floor[# / 2^12]]],
    BitwiseOr[0x80, BitwiseAnd[0x3f, Floor[# / 2^6]]],
    BitwiseOr[0x80, BitwiseAnd[0x3f, Floor[#]]]],

  # =< 0x7FFFFFFF,		Null[
    BitwiseOr[0xfc, BitwiseAnd[0x01, Floor[# / 2^30]]],
    BitwiseOr[0x80, BitwiseAnd[0x3f, Floor[# / 2^24]]],
    BitwiseOr[0x80, BitwiseAnd[0x3f, Floor[# / 2^18]]],
    BitwiseOr[0x80, BitwiseAnd[0x3f, Floor[# / 2^12]]],
    BitwiseOr[0x80, BitwiseAnd[0x3f, Floor[# / 2^6]]],
    BitwiseOr[0x80, BitwiseAnd[0x3f, Floor[#]]]],

  True,				Null[]]&, {u}]];

With[{BOM = "\uFEFF", MOB = "\uFFFE",
  AdobeSymbolMap = {
    0x20->0x0020, 	(* SPACE *)
    0x21->0x0021, 	(* EXCLAMATION MARK *)
    0x23->0x0023, 	(* NUMBER SIGN *)
    0x25->0x0025, 	(* PERCENT SIGN *)
    0x26->0x0026, 	(* AMPERSAND *)
    0x28->0x0028, 	(* OPENING PARENTHESIS *)
    0x29->0x0029, 	(* CLOSING PARENTHESIS *)
    0x2B->0x002B, 	(* PLUS SIGN *)
    0x2C->0x002C, 	(* COMMA *)
    0x2E->0x002E, 	(* PERIOD *)
    0x2F->0x002F, 	(* SLASH *)
    0x30->0x0030, 	(* DIGIT ZERO *)
    0x31->0x0031, 	(* DIGIT ONE *)
    0x32->0x0032, 	(* DIGIT TWO *)
    0x33->0x0033, 	(* DIGIT THREE *)
    0x34->0x0034, 	(* DIGIT FOUR *)
    0x35->0x0035, 	(* DIGIT FIVE *)
    0x36->0x0036, 	(* DIGIT SIX *)
    0x37->0x0037, 	(* DIGIT SEVEN *)
    0x38->0x0038, 	(* DIGIT EIGHT *)
    0x39->0x0039, 	(* DIGIT NINE *)
    0x3A->0x003A, 	(* COLON *)
    0x3B->0x003B, 	(* SEMICOLON *)
    0x3C->0x003C, 	(* LESS-THAN SIGN *)
    0x3D->0x003D, 	(* EQUALS SIGN *)
    0x3E->0x003E, 	(* GREATER-THAN SIGN *)
    0x3F->0x003F, 	(* QUESTION MARK *)
    0x5B->0x005B, 	(* OPENING SQUARE BRACKET *)
    0x5D->0x005D, 	(* CLOSING SQUARE BRACKET *)
    0x5F->0x005F, 	(* SPACING UNDERSCORE *)
    0x7B->0x007B, 	(* OPENING CURLY BRACKET *)
    0x7C->0x007C, 	(* VERTICAL BAR *)
    0x7D->0x007D, 	(* CLOSING CURLY BRACKET *)
    0xD3->0x00A9, 	(* COPYRIGHT SIGN *)
    0xE3->0x00A9, 	(* COPYRIGHT SIGN *)
    0xD8->0x00AC, 	(* NOT SIGN *)
    0xD2->0x00AE, 	(* REGISTERED TRADE MARK SIGN *)
    0xE2->0x00AE, 	(* REGISTERED TRADE MARK SIGN *)
    0xB0->0x00B0, 	(* DEGREE SIGN *)
    0xB1->0x00B1, 	(* PLUS-OR-MINUS SIGN *)
    0xB4->0x00D7, 	(* MULTIPLICATION SIGN *)
    0xB8->0x00F7, 	(* DIVISION SIGN *)
    0xA6->0x0192, 	(* LATIN SMALL LETTER SCRIPT F *)
    0x41->0x0391, 	(* GREEK CAPITAL LETTER ALPHA *)
    0x42->0x0392, 	(* GREEK CAPITAL LETTER BETA *)
    0x47->0x0393, 	(* GREEK CAPITAL LETTER GAMMA *)
!    0x44->0x2206, 	(* INCREMENT *)
    0x44->0x0394, 	(* GREEK CAPITAL LETTER DELTA *)
    0x45->0x0395, 	(* GREEK CAPITAL LETTER EPSILON *)
    0x5A->0x0396, 	(* GREEK CAPITAL LETTER ZETA *)
    0x48->0x0397, 	(* GREEK CAPITAL LETTER ETA *)
    0x51->0x0398, 	(* GREEK CAPITAL LETTER THETA *)
    0x49->0x0399, 	(* GREEK CAPITAL LETTER IOTA *)
    0x4B->0x039A, 	(* GREEK CAPITAL LETTER KAPPA *)
    0x4C->0x039B, 	(* GREEK CAPITAL LETTER LAMBDA *)
    0x4D->0x039C, 	(* GREEK CAPITAL LETTER MU *)
    0x4E->0x039D, 	(* GREEK CAPITAL LETTER NU *)
    0x58->0x039E, 	(* GREEK CAPITAL LETTER XI *)
    0x4F->0x039F, 	(* GREEK CAPITAL LETTER OMICRON *)
    0x50->0x03A0, 	(* GREEK CAPITAL LETTER PI *)
    0x52->0x03A1, 	(* GREEK CAPITAL LETTER RHO *)
    0x53->0x03A3, 	(* GREEK CAPITAL LETTER SIGMA *)
    0x54->0x03A4, 	(* GREEK CAPITAL LETTER TAU *)
    0x55->0x03A5, 	(* GREEK CAPITAL LETTER UPSILON *)
    0x46->0x03A6, 	(* GREEK CAPITAL LETTER PHI *)
    0x43->0x03A7, 	(* GREEK CAPITAL LETTER CHI *)
    0x59->0x03A8, 	(* GREEK CAPITAL LETTER PSI *)
!    0x57->0x2126, 	(* OHM *)
    0x57->0x03A9, 	(* GREEK CAPITAL LETTER OMEGA *)
    0x61->0x03B1, 	(* GREEK SMALL LETTER ALPHA *)
    0x62->0x03B2, 	(* GREEK SMALL LETTER BETA *)
    0x67->0x03B3, 	(* GREEK SMALL LETTER GAMMA *)
    0x64->0x03B4, 	(* GREEK SMALL LETTER DELTA *)
    0x65->0x03B5, 	(* GREEK SMALL LETTER EPSILON *)
    0x7A->0x03B6, 	(* GREEK SMALL LETTER ZETA *)
    0x68->0x03B7, 	(* GREEK SMALL LETTER ETA *)
    0x71->0x03B8, 	(* GREEK SMALL LETTER THETA *)
    0x69->0x03B9, 	(* GREEK SMALL LETTER IOTA *)
    0x6B->0x03BA, 	(* GREEK SMALL LETTER KAPPA *)
    0x6C->0x03BB, 	(* GREEK SMALL LETTER LAMBDA *)
    0x6D->0x03BC, 	(* GREEK SMALL LETTER MU *)
    0x6E->0x03BD, 	(* GREEK SMALL LETTER NU *)
    0x78->0x03BE, 	(* GREEK SMALL LETTER XI *)
    0x6F->0x03BF, 	(* GREEK SMALL LETTER OMICRON *)
    0x70->0x03C0, 	(* GREEK SMALL LETTER PI *)
    0x72->0x03C1, 	(* GREEK SMALL LETTER RHO *)
    0x56->0x03C2, 	(* GREEK SMALL LETTER FINAL SIGMA *)
    0x73->0x03C3, 	(* GREEK SMALL LETTER SIGMA *)
    0x74->0x03C4, 	(* GREEK SMALL LETTER TAU *)
    0x75->0x03C5, 	(* GREEK SMALL LETTER UPSILON *)
    0x66->0x03C6, 	(* GREEK SMALL LETTER PHI *)
    0x63->0x03C7, 	(* GREEK SMALL LETTER CHI *)
    0x79->0x03C8, 	(* GREEK SMALL LETTER PSI *)
    0x77->0x03C9, 	(* GREEK SMALL LETTER OMEGA *)
    0x4A->0x03D1, 	(* GREEK SMALL LETTER SCRIPT THETA *)
    0xA1->0x03D2, 	(* GREEK CAPITAL LETTER UPSILON HOOK *)
    0x6A->0x03D5, 	(* GREEK SMALL LETTER SCRIPT PHI *)
    0x76->0x03D6, 	(* GREEK SMALL LETTER OMEGA PI *)
    0xB7->0x2022, 	(* BULLET *)
    0xBC->0x2026, 	(* HORIZONTAL ELLIPSIS *)
    0xA2->0x2032, 	(* PRIME *)
    0xB2->0x2033, 	(* DOUBLE PRIME *)
    0x60->0x203E, 	(* SPACING OVERSCORE *)
!    0xA4->0x2215, 	(* DIVISION SLASH *)
    0xA4->0x2044, 	(* FRACTION SLASH *)
    0xC1->0x2111, 	(* BLACK-LETTER I *)
    0xC3->0x2118, 	(* SCRIPT P *)
    0xC2->0x211C, 	(* BLACK-LETTER R *)
    0xD4->0x2122, 	(* TRADEMARK *)
    0xE4->0x2122, 	(* TRADEMARK *)
    0xC0->0x2135, 	(* FIRST TRANSFINITE CARDINAL *)
    0xAC->0x2190, 	(* LEFT ARROW *)
    0xAD->0x2191, 	(* UP ARROW *)
    0xAE->0x2192, 	(* RIGHT ARROW *)
    0xAF->0x2193, 	(* DOWN ARROW *)
    0xAB->0x2194, 	(* LEFT RIGHT ARROW *)
    0xBF->0x21B5, 	(* DOWN ARROW WITH CORNER LEFT *)
    0xDC->0x21D0, 	(* LEFT DOUBLE ARROW *)
    0xDD->0x21D1, 	(* UP DOUBLE ARROW *)
    0xDE->0x21D2, 	(* RIGHT DOUBLE ARROW *)
    0xDF->0x21D3, 	(* DOWN DOUBLE ARROW *)
    0xDB->0x21D4, 	(* LEFT RIGHT DOUBLE ARROW *)
    0x22->0x2200, 	(* FOR ALL *)
    0xB6->0x2202, 	(* PARTIAL DIFFERENTIAL *)
    0x24->0x2203, 	(* THERE EXISTS *)
    0xC6->0x2205, 	(* EMPTY SET *)
    0xD1->0x2207, 	(* NABLA *)
    0xCE->0x2208, 	(* ELEMENT OF *)
    0xCF->0x2209, 	(* NOT AN ELEMENT OF *)
    0x27->0x220B, 	(* CONTAINS AS MEMBER *)
    0xD5->0x220F, 	(* N-ARY PRODUCT *)
    0xE5->0x2211, 	(* N-ARY SUMMATION *)
    0x2D->0x2212, 	(* MINUS SIGN *)
    0x2A->0x2217, 	(* ASTERISK OPERATOR *)
    0xD6->0x221A, 	(* SQUARE ROOT *)
    0xB5->0x221D, 	(* PROPORTIONAL TO *)
    0xA5->0x221E, 	(* INFINITY *)
    0xD0->0x2220, 	(* ANGLE *)
    0xD9->0x2227, 	(* LOGICAL AND *)
    0xDA->0x2228, 	(* LOGICAL OR *)
    0xC7->0x2229, 	(* INTERSECTION *)
    0xC8->0x222A, 	(* UNION *)
    0xF2->0x222B, 	(* INTEGRAL *)
    0x5C->0x2234, 	(* THEREFORE *)
    0x7E->0x223C, 	(* TILDE OPERATOR *)
    0x40->0x2245, 	(* APPROXIMATELY EQUAL TO *)
    0xBB->0x2248, 	(* ALMOST EQUAL TO *)
    0xB9->0x2260, 	(* NOT EQUAL TO *)
    0xBA->0x2261, 	(* IDENTICAL TO *)
    0xA3->0x2264, 	(* LESS THAN OR EQUAL TO *)
    0xB3->0x2265, 	(* GREATER THAN OR EQUAL TO *)
    0xCC->0x2282, 	(* SUBSET OF *)
    0xC9->0x2283, 	(* SUPERSET OF *)
    0xCB->0x2284, 	(* NOT A SUBSET OF *)
    0xCD->0x2286, 	(* SUBSET OF OR EQUAL TO *)
    0xCA->0x2287, 	(* SUPERSET OF OR EQUAL TO *)
    0xC5->0x2295, 	(* CIRCLED PLUS *)
    0xC4->0x2297, 	(* CIRCLED TIMES *)
    0x5E->0x22A5, 	(* UP TACK *)
    0xD7->0x22C5, 	(* DOT OPERATOR *)
    0xF3->0x2320, 	(* TOP HALF INTEGRAL *)
    0xF5->0x2321, 	(* BOTTOM HALF INTEGRAL *)
    0xE1->0x2329, 	(* BRA *)
    0xF1->0x232A, 	(* KET *)
!    0xE0->0x25CA, 	(* LOZENGE *)
    0xE0->0x22C4, 	(* DIAMOND OPERATOR *)
    0xAA->0x2660, 	(* BLACK SPADE SUIT *)
    0xA7->0x2663, 	(* BLACK CLUB SUIT *)
    0xA9->0x2665, 	(* BLACK HEART SUIT *)
    0xA8->0x2666, 	(* BLACK DIAMOND SUIT *)
    Null[]}},

  (* Convert AdobeSymbol code-points into Unicode code-points *)
  Apply[(AdobeSymbol2Unicode[#1] = #2)&, AdobeSymbolMap, {1}];
  AdobeSymbol2Unicode[c_Real] := c;

  (* Convert AdobeSymbol code-points into UTF-8 String *)
  AdobeSymbol2UTF8[c___Real] := Unicode2UTF8@@Map[AdobeSymbol2Unicode, {c}];

  AdobeSymbol2UTF8[s_String] := If[StringPosition[s, "`f", 1] === {}, s,
    StringJoin@@Module[{utf8 = False}, Map[Switch[{#[1], #[2]},
      {"`", "f"},	If[utf8,
	"`f"//AdobeSymbol2UTF8@@ToCharacterCode[#[3, -1]],
	utf8 = True; "`f"//BOM//AdobeSymbol2UTF8@@ToCharacterCode[#[3, -1]]],
      {"`", "n"},	If[utf8,
	utf8 = False; MOB//#,
	#],
      {"`", _},		If[utf8,
	#[1, 2]//AdobeSymbol2UTF8@@ToCharacterCode[#[3, -1]],
	#],
      _,		#]&, Apply[s, Partition[Flatten[{1, Module[{prev = 0},
	Map[If[# =< prev, Null[], (prev = # + 1; {# - 1, #})]&, StringPosition[s, "`"][[, 1]]]], -1}], 2], {1}]]]];
  ];

Protect[Unicode2UTF8, AdobeSymbol2Unicode, AdobeSymbol2UTF8];

! End of File
